# Capture S88 Sensor Data from an Arduino Serial Transmission
# In the form: Character 'Q' or 'q', with Q=Sensor ON and q=Sensor OFF
# followed by 2 bytes: sensor # 1-512 with 1st byte=MSB and 2nd byte=LSB
# Author: Philippe Chavatte 2019 - lormedy@free.fr based in part
# on a script by Bob Jacobsen as part of the JMRI distribution
# Version 1.4
# An Automat object to create a separate thread
# that can sit there, waiting for each character to arrive

import jarray
import jmri
import purejavacomm

class SerialSensorMux(jmri.jmrit.automat.AbstractAutomaton) :
    # ctor starts up the serial port
    def __init__(self, portname) :
        global extport
        self.portID = purejavacomm.CommPortIdentifier.getPortIdentifier(portname)
        try:
            self.port = self.portID.open("JMRI", 50)
        except purejavacomm.PortInUseException:
            self.port = extport
        extport = self.port
        # set options on port
        baudrate = 115200
        self.port.setSerialPortParams(baudrate,
            purejavacomm.SerialPort.DATABITS_8,
            purejavacomm.SerialPort.STOPBITS_1,
            purejavacomm.SerialPort.PARITY_NONE)
        # Anticipate the Port Opening will restart the Arduino
        # Uncomment the following line for use on RPi You may need to fiddle with the 2000
        # msec delay this works on RPi 3 B+
        # self.waitMsec(2000)
        # get I/O connections for later
        self.inputStream = self.port.getInputStream()
        self.outputStream = self.port.getOutputStream()
        return

    # init() is the place for your initialization
    def init(self) :
    # write(self, "<Y S88bus_Size DataFormat>") # S88bus_Size = group of 8 sensors, maxi = 64 (64*8 = 512 sensors)
            # DataFormat: 0 = binAscii, 1 = hexAscii, 2 =  = purehexa, 3 = sensor format (Q ID & q ID)
            # data is transmitted at each sensor state change
        <Q> # use <Y 64 3> or <Q>
        return

    # handle() is called repeatedly until it returns false.
    def handle(self) :
        global ttest
        if ttest == 1 :
            self.outputStream.write('!')
            self.outputStream.write('!')
            self.outputStream.write('!')
            self.outputStream.write(0x0D)
            ttest = 0
        # get next character
        sensor_state = self.inputStream.read()
        if (sensor_state & 223) != 'Q' :
                return 1
        sensor_state = ( sensor_state >> 5 ) & 1
        sensor_num_high = self.inputStream.read()
        sensor_num = self.inputStream.read()
        sensor_num = (sensor_num_high << 16) + (sensor_num & 0xFF)
        mesg = "AR:%d" % (sensor_num)
        if sensor_num == 511 and sensor_state == 1 :
            print "Sensor Read Ends"
            self.inputStream.close()
            self.outputStream.close()
            self.port.close()
            return 0
        s = sensors.getByUserName( mesg )
        if s is None :
            print mesg, " Not Available"
            return 1
        if sensor_state == 1 :
            s.setKnownState(ACTIVE)
        if sensor_state == 0 :
            s.setKnownState(INACTIVE)

        return 1    # to continue, 0 to Kill Script

    def write(self, data) :
        # now send
        self.outputStream.write(data)
        return

    def flush(self) :
        self.outputStream.flush()
        return
ttest=1
# create one of these; provide the name of the serial port
a = SerialSensorMux("COM5")

# set the thread name, so easy to cancel if needed
a.setName("SerialSensorMux script")

# start running
a.start();

# setup complete
a.flush()
